require 'rubygems'
require 'gosu'
                        # Generic ingredient class

require 'burger_ingredient.rb'

module RubyBurger
  class FallingIngredient
    FALL_SPEED = 5

    def initialize(window, type, x, y)
      @window = window
      @type = type
      @x = x
      @y = y
                        # A png image should exist for parameter "type"
      @image = Gosu::Image.new(@window, IMG_PATH + 'burger/' + @type + '.png', false)
    end

    def update
                        # Test if this item should be stacked on top of the burger
      $bb = @window.top_ingredient
                        # Is it position properly?
      if (@x + (width / 2)).between?($bb.x, $bb.x + $bb.width) and (@y.between?($bb.y - 20, $bb.y + 20))
        $newbi = BurgerIngredient.new(@window, @type)
        $newbi.update
        @window.add($newbi)
        delete
      end

                        # Fall
      step_fall
    end

    def draw
                        # Draw
      @image.draw(@x, @y, 2)
    end
                        # Gradually fall down the screen
    def step_fall
      @y += FALL_SPEED
    end

    def delete
                        # Delete this ingredient
      @window.delete_falling_ingredient(self)
    end

    def width
      @image.width
    end

    def height
      @image.height
    end

    def x
      @x
    end

    def y
      @y
    end
  end
                        # A special ingredient: the bottom bun! It follows the mouse's x position
  class BottomBun
    def initialize(window)
      @window = window
      @x = 0
      @y = 0
      @image = Gosu::Image.new(@window, IMG_PATH + "burger/bottom_bun.png", false)
                        # Sets how many ingredients high the burger can be stacked before it starts scrolling down
      @stack_limit = 10
    end

    def draw
                        # Draw
      @image.draw(@x, @y, 2)
    end

    def update
                        # Follow the mouse's x position
      @x = mouse_x - (width / 2)

                        # Don't let it go off the screen
      if @x < 0
        @x = 0
      end

      if (@x + width) > (@window.width)
        @x = @window.width - width
      end
    end

    def mouse_x
      @window.mouse_x
    end

    def update_y(stack)
                        # Figure out the correct y position. After all, everything else on the burger relies on it
      @y = @window.height - height
      $l = (stack.length - 1) - @stack_limit
      if $l < 0
        $l = 0
      end
      @y += ($l * height)
    end

    def width
      @image.width
    end

    def height
      @image.height
    end

    def x
      @x
    end

    def y
      @y
    end
  end
end